#!/usr/bin/python3

import os
import fcntl
import shutil
import logging
from gettext import gettext as _

from SystemUpdater.Core.errors import *
from SystemUpdater.Core.enums import *

class InhibitShutdownLock():
    # 禁止关机锁文件路径
    FILELOCK_PATH = "/tmp/lock/"
    SHUTDOWN_BLOCK_FILELOCK = "kylin-update.lock"
    LOGIND_SLEEP_CONF = "/etc/systemd/sleep.conf.d/"
    LOGIND_SLEEP_POSITION = "/etc/systemd/sleep.conf.d/inhibit-sleep.conf"
    SLEEP_CONF_POSITION = "/var/lib/kylin-system-updater/inhibit-sleep.conf"
    NEED_ROLLBACK_FALG = "/var/lib/kylin-system-updater/need-rollback"

    UPDATE_NOT_RESOTRE_DIR = "/var/lib/kylin-system-updater/remain"
    EXCEPTION_SHUTDOWN_FLAG = "/var/lib/kylin-system-updater/remain/install-exception-flag"

    def __init__(self):
        self.inhibit_lock = None
        self.pidfile = None

    #安装时禁止关机 进行加锁
    def lock(self, caller='kylin-system-updater',inhibit_sleep = False):
        """
        Send a dbus signal to logind to not suspend the system, it will be
        released when the return value drops out of scope
        """
        self._make_execption_check_flag()

        if inhibit_sleep == True:
            self._lock_sleep()

        if self.inhibit_lock == None:
            try:
                from gi.repository import Gio, GLib
                logging.info("Start acquiring Shutdown lock(block)...")
                connection = Gio.bus_get_sync(Gio.BusType.SYSTEM)
                
                var, fdlist = connection.call_with_unix_fd_list_sync(
                    'org.freedesktop.login1', '/org/freedesktop/login1',
                    'org.freedesktop.login1.Manager', 'Inhibit',
                    GLib.Variant('(ssss)',
                                ('shutdown:sleep',
                                caller, _("Installing or removing software packages."),
                                'block')),
                    None, 0, -1, None, None)
                self.inhibit_lock = Gio.UnixInputStream(fd=fdlist.steal_fds()[var[0]])
                self.LockedPreventShutdown()
                
                logging.info("Shutdown Has been locked...")
                return True
            except Exception as e:
                logging.error(e)
                return False
        else:
            logging.info("No need to acquired lock and Shutdown Has been locked...")
            return False

    #解锁禁止关机 
    def unlock(self):
        try:
            self._remove_check_flag()
            self.unLockedEnableShutdown()
            self._unlock_sleep()
            if self.inhibit_lock != None:
                self.inhibit_lock.close()
                self.inhibit_lock = None
                logging.info("Shutdown Has been unlocked...")
            else:
                logging.info("Not locked and Quitting ...")
        except Exception as e:
            self.inhibit_lock = None
            logging.error("unlock failed." + str(e))

    def _lock_sleep(self):
        try:
            if not os.path.exists(self.LOGIND_SLEEP_CONF):
                logging.info("File(%s) is not exists and will be create",self.LOGIND_SLEEP_CONF)
                os.makedirs(self.LOGIND_SLEEP_CONF)
            shutil.copy(self.SLEEP_CONF_POSITION, self.LOGIND_SLEEP_CONF)
            logging.info("File(%s) will be create for forcely inhibit sleep...",self.LOGIND_SLEEP_CONF)
        except Exception as e:
            logging.error(e)
            
    def _unlock_sleep(self):
        try:
            if os.path.exists(self.LOGIND_SLEEP_POSITION):
                os.remove(self.LOGIND_SLEEP_POSITION)
                # logging.info("File(%s) is exists and will be delete and unlock sleep",self.LOGIND_SLEEP_POSITION)
        except Exception as e:
            logging.error(e)
    
    @staticmethod
    def _check_unlock_sleep():
        try:
            LOGIND_SLEEP_POSITION = "/etc/systemd/sleep.conf.d/inhibit-sleep.conf"
            NEED_ROLLBACK_FALG = "/var/lib/kylin-system-updater/need-rollback"

            if os.path.exists(LOGIND_SLEEP_POSITION):
                os.remove(LOGIND_SLEEP_POSITION)
                logging.warning("File(%s) is exists and will be delete and unlock sleep",LOGIND_SLEEP_POSITION)

            if os.path.exists(NEED_ROLLBACK_FALG):
                os.remove(NEED_ROLLBACK_FALG)
                logging.warning("File(%s) is exists and will be delete and unlock restore",NEED_ROLLBACK_FALG)
        except Exception as e:
            logging.error(e)

    def mark_restore(self):
        with open("/var/lib/kylin-system-updater/need-rollback", 'w') as f:
            pass

    def remove_restore_flag(self):
        if os.path.exists(self.NEED_ROLLBACK_FALG):
            os.remove(self.NEED_ROLLBACK_FALG)

    #安装时禁止关机 进行加锁
    def LockedPreventShutdown(self):
        #不为空是表示以及被锁
        if self.pidfile != None:
            logging.error("self.pidfile file disc not is None,Has been locked...")
            return False

        if not os.path.exists(self.FILELOCK_PATH):
            #不存在创建
            logging.info("File(%s) is not exists and will be create",self.FILELOCK_PATH)
            os.makedirs(self.FILELOCK_PATH)
        else:
            #当目录存在时进行删除 不删除进行创建文件的话会报错 
            # file cannot be locked.[Errno 11] Resource temporarily unavailable
            # 资源被占用报错
            shutil.rmtree(self.FILELOCK_PATH)
            logging.info("File(%s) is exists and will be delete and create",self.FILELOCK_PATH)
            os.makedirs(self.FILELOCK_PATH)        

        try:
            self.pidfile = open(os.path.join(self.FILELOCK_PATH, self.SHUTDOWN_BLOCK_FILELOCK), "w+")
            fcntl.flock(self.pidfile, fcntl.LOCK_EX | fcntl.LOCK_NB)
            return True
        except Exception as e:
            logging.error("file cannot be locked." + str(e))
            self.pidfile = None
            return False

    #解锁禁止关机 
    def unLockedEnableShutdown(self):
        #未加锁退出
        if not self.pidfile:
            logging.info("Not locked and Quitting ...")
            return False
        try:
            fcntl.flock(self.pidfile, fcntl.LOCK_UN)
            self.pidfile.close()
            self.pidfile = None

            # Fix 修复权限问题 当普通用户无法使用 所以直接删除目录
            if os.path.exists(self.FILELOCK_PATH):
                shutil.rmtree(self.FILELOCK_PATH)
                logging.info('Emptying the lockPath(%s) is complete...',self.FILELOCK_PATH)
            else:
                logging.info("Emptying the lockPath(%s) is Failed...",self.FILELOCK_PATH)

            return True
        except Exception as e:
            logging.error("unlock failed." + str(e))
            self.pidfile = None
            return False

    def _make_execption_check_flag(self):
        if not os.path.exists(self.UPDATE_NOT_RESOTRE_DIR):
            os.makedirs(self.UPDATE_NOT_RESOTRE_DIR)

        with open(self.EXCEPTION_SHUTDOWN_FLAG, 'w') as file:
            pass  # 不写入任何内容，只创建文件
    
    def _remove_check_flag(self):
        try:
            os.remove(self.EXCEPTION_SHUTDOWN_FLAG)
        except Exception as e:
            pass
    
    # 存在异常时 检查到时 获取完状态直接删除
    def is_abnormal(self):
        if os.path.exists(self.EXCEPTION_SHUTDOWN_FLAG):
            self._remove_check_flag()
            return True
        else:
            return False
