#!/bin/bash

# set -x 

# sh -c "$(curl -fksSL https://172.20.191.209/wangsong/analyse-problem/-/raw/master/analyse-error.sh)"

APPNAME=$(basename "$0" | sed "s/\.sh$//")

# -----------------------------------------------------------------------------
# Log functions
# -----------------------------------------------------------------------------

fn_log_info()  { echo "$APPNAME: $1"; }
fn_log_warn()  { echo "$APPNAME: [WARNING] $1" 1>&2; }
fn_log_error() { echo "$APPNAME: [ERROR] $1" 1>&2; }

fn_error_desc() { echo "错误描述：$1" 1>&2; }
fn_solution_desc() { echo "解决方案：$1" 1>&2; }

fn_log_info_cmd()  {
	if [ -n "$SSH_DEST_FOLDER_PREFIX" ]; then
		echo "$APPNAME: $SSH_CMD '$1'";
	else
		echo "$APPNAME: $1";
	fi
}

#检查命令是否存在
command_exists() {
	command -v "$@" >/dev/null 2>&1
}

if [ -t 1 ]; then
  is_tty() {
    true
  }
else
  is_tty() {
    false
  }
fi

supports_truecolor() {
  case "$COLORTERM" in
  truecolor|24bit) return 0 ;;
  esac

  case "$TERM" in
  iterm           |\
  tmux-truecolor  |\
  linux-truecolor |\
  xterm-truecolor |\
  screen-truecolor) return 0 ;;
  esac

  return 1
}

setup_color() {
  if ! is_tty; then
    FMT_RAINBOW=""
    FMT_RED=""
    FMT_GREEN=""
    FMT_YELLOW=""
    FMT_BLUE=""
    FMT_BOLD=""
    FMT_RESET=""
    return
  fi

  if supports_truecolor; then
    FMT_RAINBOW="
      $(printf '\033[38;2;255;0;0m')
      $(printf '\033[38;2;255;97;0m')
      $(printf '\033[38;2;247;255;0m')
      $(printf '\033[38;2;0;255;30m')
      $(printf '\033[38;2;77;0;255m')
      $(printf '\033[38;2;168;0;255m')
      $(printf '\033[38;2;245;0;172m')
    "
  else
    FMT_RAINBOW="
      $(printf '\033[38;5;196m')
      $(printf '\033[38;5;202m')
      $(printf '\033[38;5;226m')
      $(printf '\033[38;5;082m')
      $(printf '\033[38;5;021m')
      $(printf '\033[38;5;093m')
      $(printf '\033[38;5;163m')
    "
  fi

  FMT_RED=$(printf '\033[31m')
  FMT_GREEN=$(printf '\033[32m')
  FMT_YELLOW=$(printf '\033[33m')
  FMT_BLUE=$(printf '\033[34m')
  FMT_BOLD=$(printf '\033[1m')
  FMT_RESET=$(printf '\033[0m')
}


# -----------------------------------------------------------------------------
# Make sure everything really stops when CTRL+C is pressed
# -----------------------------------------------------------------------------

fn_terminate_script() {
	fn_log_info "SIGINT caught."
	exit 1
}
trap 'fn_terminate_script' SIGINT


TRUE=true
FALSE=false


# UPDATER_LOG_PATH="./tests/kylin-system-updater.log.1"
# APT_TERM_LOG="./tests/install-failed-data/未签名，拒绝安装"

APT_TERM_LOG="/var/log/apt/term.log"
UPDATER_LOG_PATH="/var/log/kylin-system-updater/kylin-system-updater.log.1"

OFFLINE_SOURCE_PATH="/media/OfflineSource"

OFFLINE_SOURCE_FLAG=$FALSE

ERROR_DESC=""
SOURCE_TYPE=""
PUSH_UPDATE_PATCH=""
SOURCES_LIST=""

# 检查是否使用离线源
if [ -e "$OFFLINE_SOURCE_PATH" ]; then
    if [ -n "$(ls -A "$OFFLINE_SOURCE_PATH")" ]; then
        SOURCE_TYPE="离线源更新"
        OFFLINE_SOURCE_FLAG=$TRUE
    else
        SOURCE_TYPE="在线源更新"
    fi
else
    SOURCE_TYPE="在线源更新"
fi

# 检查推送内容
if [ -f "/var/lib/kylin-software-properties/template/important.list" ]; then
    if [ -s "/var/lib/kylin-software-properties/template/important.list" ]; then
        PUSH_UPDATE_PATCH=$(cat /var/lib/kylin-software-properties/template/important.list)
    else
        PUSH_UPDATE_PATCH=""
    fi
else
    PUSH_UPDATE_PATCH=""
fi

# 检查源信息
if [ -f "/etc/kylin-system-updater/apt_private.conf" ]; then
    SOURCES_LIST="/var/lib/kylin-system-updater/sources.list"
else
    SOURCES_LIST="/etc/apt/sources.list"
fi

fn_analyse_log() {
    # 提取日志最后的错误信息进行分析
    bug_msg=$(grep 'UpdateDetectFinished success = False \|UpdateInstallFinished success = False' $UPDATER_LOG_PATH  | tail -n 1)
    error_desc=$(echo $bug_msg  | grep -o 'error_desc = [^ ]*' | sed 's/error_desc = //')

    if [ -z "$error_desc" ]; then
        echo "未检查到日志有相关错误信息..."
        exit 100
    fi    
    echo $error_desc
}

fn_apt_update() {
    if [ -f "/etc/kylin-system-updater/apt_private.conf" ]; then
        sudo apt update -c /etc/kylin-system-updater/apt_private.conf
	else
        sudo apt update
    fi
}

fn_apt_download() {
    if [ -f "/etc/kylin-system-updater/apt_private.conf" ]; then
        apt download $1 -c /etc/kylin-system-updater/apt_private.conf
	else
        apt download $1
    fi
}

fn_apt_install() {
    if [ -f "/etc/kylin-system-updater/apt_private.conf" ]; then
        sudo apt install --simulate $1 -c /etc/kylin-system-updater/apt_private.conf
	else
        sudo apt install --simulate $1
    fi
}

fn_error_print() {
    echo "发现下面错误信息，详细错误请查看/var/log/apt/term.log日志:"
    echo ""
    echo "$1" | grep "$2"
    echo ""
}

fn_install_failed() {
    INSTALL_FAILED_SAFE="验证失败，拒绝安装！"
    INSTALL_FAILED_SAFE_OTHER="未签名，拒绝安装！"

    INSTALL_FAILED_SCRIPT="子进程返回错误状态"
    INSTALL_FAILED_SCRIPT_EN="script subprocess returned error"

    INSTALL_FAILED_INCLUDE="同时被包含于软件包"
    NSTALL_FAILED_INCLUDE_EN="which is also in package"

    SAFE_WHITELIST_FAILED="in the prevent uninstallation of whitelist!"

    term_msg=$(tac "$APT_TERM_LOG" | awk '/Log ended:/,/Log started:/ {if ($0 ~ /Log started:/ && !found) found=1; if (!found) print}' | tac)

    if [[ $term_msg == *"$INSTALL_FAILED_SAFE"* || $term_msg == *"$INSTALL_FAILED_SAFE_OTHER"* ]]; then
        fn_error_print "$term_msg" "$INSTALL_FAILED_SAFE \|$INSTALL_FAILED_SAFE_OTHER"
        
        fn_error_desc "安全验证错误"
        fn_solution_desc "请联系 安全研发人员"
        exit 0
    # 安装前后脚本执行错误
    elif [[ $term_msg == *"$INSTALL_FAILED_SCRIPT"* || $term_msg == *"$INSTALL_FAILED_SCRIPT_EN"* ]]; then
        echo "发现下面错误信息，详细错误请查看/var/log/apt/term.log日志:"
        echo ""
        echo "$term_msg" | grep -B 5 "$INSTALL_FAILED_SCRIPT \|$INSTALL_FAILED_SCRIPT_EN"
        echo ""
        fn_error_desc "软件包安装前后脚本执行错误。"        
        fn_solution_desc "请联系上述的软件包维护人。"
        exit 0
    elif [[ $term_msg == *"$INSTALL_FAILED_INCLUDE"* || $term_msg == *"$NSTALL_FAILED_INCLUDE_EN"* ]]; then
        echo "发现下面错误信息，详细错误请查看/var/log/apt/term.log日志:"
        echo ""
        echo "$term_msg" | grep -B 1 "$INSTALL_FAILED_INCLUDE \|$NSTALL_FAILED_INCLUDE_EN"
        
        fn_error_desc "两个软件包，包含相同文件。"
        fn_solution_desc "请联系上述的软件包维护人。"
        exit 0
    elif [[ $term_msg == *"$SAFE_WHITELIST_FAILED"* ]]; then
        fn_error_print "$term_msg" "$SAFE_WHITELIST_FAILED"
        
        fn_error_desc "安全验证错误"
        fn_solution_desc "请联系 安全研发人员"
        exit 0
    else
        echo "无法识别或者没有错误"
    fi    
}

fn_install_failed_easy() {
    INSTALL_FAILED_SAFE="验证失败，拒绝安装！"

    INSTALL_FAILED_SCRIPT="子进程返回错误状态"
    INSTALL_FAILED_SCRIPT_EN="script subprocess returned error"

    INSTALL_FAILED_INCLUDE="同时被包含于软件包"

    SAFE_WHITELIST_FAILED="in the prevent uninstallation of whitelist!"

    term_msg=$(tac "$APT_TERM_LOG" | awk '/Log ended:/,/Log started:/ {if ($0 ~ /Log started:/ && !found) found=1; if (!found) print}' | tac)

    if [[ $term_msg == *"$INSTALL_FAILED_SAFE"* ]]; then
        echo "验证失败，拒绝安装"
        exit 0
    # 安装前后脚本执行错误
    elif [[ $term_msg == *"$INSTALL_FAILED_SCRIPT"* || $term_msg == *"$INSTALL_FAILED_SCRIPT_EN"* ]]; then
        echo "安装前后脚本执行错误"
        exit 0
    elif [[ $term_msg == *"$INSTALL_FAILED_INCLUDE"* ]]; then
        echo "两个软件包同时包含相同文件"
        exit 0
    elif [[ $term_msg == *"$SAFE_WHITELIST_FAILED"* ]]; then
        echo "卸载白名单"
        exit 0
    else
        echo "无法识别或者没有错误"
    fi    
}

fn_self_depends() {
# 优先升级依赖问题

    echo ""
}

fn_check_update_source() {
    # 检查是否采用A/B源机制
    fn_apt_update

    if ! [ $? -eq 0 ]; then
        echo "源更新失败，检查1、使用的源是否正确，2、检查host的配置"
        echo "若其他设备都存在问题，则联系版本组检查源"
    else
        echo "执行report-updater-bug导出日志放入禅道，指派给系统更新研发人员"
    fi
}

fn_check_latest() {
    # 检查推送文件是否存在
    if [ -f "/var/lib/kylin-software-properties/template/important.list" ]; then
        # 检查内容是否为空
        if [ -s "/var/lib/kylin-software-properties/template/important.list" ]; then
            echo "推送内容为："
            cat /var/lib/kylin-software-properties/template/important.list

            if ! [ -f "/usr/share/kylin-update-desktop-config/config/kylin-update-desktop-system.json" ]; then
                echo "联系版本组解决，没有kylin-update-desktop-system.json升级内容配置"
                exit 100
            fi
            
            echo "存在推送内容，已经升级过或者推送源不匹配，请检查源是否推送正确"
            echo "若还是无法解决联系 系统更新研发"
        else
            echo "推送内容为空，请检查是否导入序列号，并且在源管理服务端推送内容"
        fi
	else
        echo "没有推送内容文件，请检查是否导入序列号"
    fi
}
fn_check_overall() {
    echo ""
}

fn_check_config_depends() {

    echo "错误描述:配置软件包kylin-update-desktop-config无法安装，软件源中不存在此包"
    echo ""
    echo "开始检查config包是否在源内"

    fn_apt_install  kylin-update-desktop-config

    if [ $? -eq 0 ]; then
        echo "下载成功，导出系统更新日志到禅道，联系系统更新研发"
    else
        echo "发现错误，未发现config在源内，请检查使用的源是否正确，或者推送是否正确"
        echo "若还是无法解决，请联系版本组解决"
    fi
}

fn_check_download_pkgs() {
# 从文件末尾开始查找匹配到的字符串
    search_string="Hashes of received file"
    match_line=$(grep -n "$search_string" "$UPDATER_LOG_PATH" | head -n 1 | cut -d ":" -f 1)

    # 如果找到匹配的行
    if [ -n "$match_line" ]; then
        # 输出匹配内容上下10行的内容
        echo "发现下面错误信息:"
        echo ""
        tail -n +$((match_line - 8)) "$UPDATER_LOG_PATH" | head -n 17

        echo "请联系版本组检查离线源或者在线源内容"
    else
        echo "当前使用在线源更新方式，测试下载软件包从源中"
        fn_apt_download kylin-system-updater
        if [ $? -eq 0 ]; then
            rm *kylin-system-updater*.deb
            echo "下载软件包正常，导出系统更新日志到禅道"
        else
            if ! [ "$OFFLINE_SOURCE_FLAG" = "$TRUE" ]; then
                echo "测试下载失败，检查网络连接，网络连接完善后使用apt download tree 软件包测试是否正常"
            else
                echo "测试下载失败，联系版本组检查离线源是否正常"
            fi
        fi
    fi
}

fn_check_source_manager() {

    echo "访问源服务器失败，不能使用源管理服务器获取相关的推送升级内容"

    echo ""

    echo "正在使用终端进行测试请稍后..."

    local opt_msg=$(time timeout 30s kylin-source-update 2>&1)

    echo "测试结果如下："

    echo $opt_msg

    echo ""
    if [[ $opt_msg == *"true"* ]]; then
        echo "导出日志，联系系统更新研发人员"
    else
        echo "执行失败，出现错误提示，联系源管理研发人员"
    fi 
}

arg_prompt(){
    cat <<EOF
系统更新相关基础信息显示,确定是否符合预期:

    1.更新方式：$SOURCE_TYPE

    2.更新内容：$PUSH_UPDATE_PATCH

    3.更新源：$SOURCES_LIST

    4.${FMT_RED}上次错误：$ERROR_DESC${FMT_RESET}
                
    5.确认：信息准确并执行错误检查? [Y/n]
EOF

	read -r opt
	case $opt in
	y*|Y*|"") ;;
	n*|N*) echo ""; exit 0 ;;
	*) echo "无效输入，请重新执行"; exit 0 ;;
	esac
}

main() {
    # clear
    setup_color

    while :; do
        case $1 in
            -h|-\?|--help)
                fn_display_usage
                exit
                ;;
            -f|--install-failed)
                shift
                fn_install_failed
                exit 0
                ;;
            -f|--install-failed-easy)
                shift
                fn_install_failed_easy
                exit 0
                ;;
            *)
                break
        esac

        shift
    done

    # 提取日志最后的错误信息进行分析
    bug_msg=$(grep 'UpdateDetectFinished success = False \|UpdateInstallFinished success = False \|UpdateDetectFinished success = True , upgrade_group = \[\]' $UPDATER_LOG_PATH  | tail -n 1)
    ERROR_DESC=$(echo $bug_msg  | grep -o 'error_desc = [^ ]*' | sed 's/error_desc = //')

    if ! [ -z "$ERROR_DESC" ]; then
        echo ""
    elif [[ $bug_msg == *"UpdateDetectFinished success = True , upgrade_group"* ]]; then
        ERROR_DESC="您的系统已是最新"
    else    
        echo "未检查到日志有相关错误信息..."
        exit 100
    fi

    arg_prompt

    case $ERROR_DESC in
        无法安装或移除软件包。|安装或移除软件包失败。)
            fn_install_failed
            ;;
        无法进行优先升级，请检查优先升级包相关依赖。)
            fn_self_depends
            ;;
        无法下载软件仓库信息，检查您的网络连接。)
            fn_check_update_source
            ;;
        您的系统已是最新)
            fn_check_latest
            ;;
        无法访问源管理服务器，请稍后再试)
            fn_check_source_manager
            ;;
        无法安装组配置包，请检查组配置包相关依赖。)
            fn_check_config_depends
            ;;
        本次更新无法检测到可升级的软件包。)
            fn_check_overall
            ;;
        检查您的网络连接。)
            fn_check_download_pkgs
            ;;
        *)
            echo "此错误目前不支持检查，后续版本支持"
    esac
}

main "$@"